@@ -31,6 +31,7 @@
 #include "nautilus-global-preferences.h"
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gnome-extensions.h>
+#include <eel/eel-vfs-extensions.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-preferences.h>
 #include <eel/eel-string.h>
@@ -38,6 +39,7 @@
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
+#include <libgnome/gnome-desktop-item.h>
 #include <libgnomeui/gnome-uidefs.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -592,64 +594,7 @@ nautilus_launch_application (GnomeVFSMimeApplication *application,
 	g_free (parameter);
 }
 
-static void
-concat_option_cb (const char *string,
-		  gpointer    callback_data)
-{
-	char *quoted;
-	GString *s;
-	
-	s = callback_data;
-	quoted = g_shell_quote (string);
 
-	g_string_append (s, quoted);
-	g_string_append_c (s, ' ');
-
-	g_free (quoted);
-}
-
-static char *
-get_xalf_prefix (const char *name)
-{
-	char *xalf_executable;
-	GString *s;
-	char *quoted;
-	EelStringList *str_list;
-
-	/* FIXME bugzilla.gnome.org 48206: At time I am writing this,
-	 * xalf is still pretty buggy, and Nautilus uses it a lot more
-	 * than the Panel does with no way to turn it off for
-	 * individual programs the way you can in the Panel. Sadly,
-	 * Ximian GNOME 1.4 ships with xalf on by default. So we did
-	 * this lame thing and turned it off unless you define this
-	 * environment variable. Some day we can remove this.
-	 */
-	if (g_getenv ("NAUTILUS_USE_XALF") == NULL) {
-		return g_strdup ("");
-	}
-	if (!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_XALF_ENABLED)) {
-		return g_strdup ("");
-	}
-	xalf_executable = g_find_program_in_path ("xalf");
-	if (xalf_executable == NULL) {
-		return g_strdup ("");
-	}
-
-	s = g_string_new (xalf_executable);
-	g_string_append (s, " --title ");
-	quoted = g_shell_quote (name);
-	g_string_append (s, quoted);
-	g_free (quoted);
-	g_string_append_c (s, ' ');
-
-	str_list = eel_preferences_get_string_list (NAUTILUS_PREFERENCES_XALF_OPTIONS);
-	if (str_list) {
-		eel_string_list_for_each (str_list, concat_option_cb, s);
-		eel_string_list_free (str_list);
-	}
-
-	return g_string_free (s, FALSE);
-}
 
 /**
  * nautilus_launch_application_from_command:
@@ -669,8 +614,6 @@ nautilus_launch_application_from_command (const char *name,
 {
 	char *full_command;
 	char *quoted_parameter; 
-	char *final_command;
-	char *xalf_prefix;
 
 	if (parameter != NULL) {
 		quoted_parameter = g_shell_quote (parameter);
@@ -680,17 +623,120 @@ nautilus_launch_application_from_command (const char *name,
 		full_command = g_strdup (command_string);
 	}
 
-	xalf_prefix = get_xalf_prefix (name);
-	final_command = g_strconcat (xalf_prefix, full_command, NULL);
-
-	g_free (full_command);
-	g_free (xalf_prefix);
-	
 	if (use_terminal) {
-		eel_gnome_open_terminal (final_command);
+		eel_gnome_open_terminal (full_command);
 	} else {
-	    	eel_gnome_shell_execute (final_command);
+	    	eel_gnome_shell_execute (full_command);
 	}
 
-	g_free (final_command);
+	g_free (full_command);
+}
+
+void
+nautilus_launch_desktop_file (const char	*desktop_file_uri,
+				const GList	*parameter_uris,
+				GtkWindow	*parent_window)
+{
+	GError *error;
+	GnomeDesktopItem *ditem;
+	const char *command_string;
+	char *local_path, *message;
+	const GList *p;
+	int total, count;
+
+	/* strip the leading command specifier */
+	if (eel_str_has_prefix (desktop_file_uri, NAUTILUS_DESKTOP_COMMAND_SPECIFIER)) {
+		desktop_file_uri += strlen (NAUTILUS_DESKTOP_COMMAND_SPECIFIER);
+	}
+
+	/* Don't allow command execution from remote locations where the
+	 * uri scheme isn't file:// (This is because files on for example
+	 * nfs are treated as remote) to partially mitigate the security
+	 * risk of executing arbitrary commands.
+	 */
+	if (!eel_vfs_has_capability (desktop_file_uri,
+				     EEL_VFS_CAPABILITY_SAFE_TO_EXECUTE)) {
+		eel_show_error_dialog
+			(_("Sorry, but you can't execute commands from "
+			   "a remote site due to security considerations."), 
+			 _("Can't execute remote links"),
+			 parent_window);
+			 
+		return;
+	}
+	
+	error = NULL;
+	ditem = gnome_desktop_item_new_from_uri (desktop_file_uri, 0,
+						&error);	
+	if (error != NULL) {
+		message = g_strconcat (_("There was an error launching the application.\n\n"
+					 "Details: "), error->message, NULL);
+		eel_show_error_dialog
+			(message,
+			 _("Error launching application"),
+			 parent_window);			
+			 
+		g_error_free (error);
+		g_free (message);
+		return;
+	}
+	
+	/* check if this app only supports local files */
+	command_string = gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_EXEC);
+	if ((strstr (command_string, "%F") || strstr (command_string, "%f"))
+		&& !(strstr (command_string, "%U") || strstr (command_string, "%u"))
+		&& parameter_uris != NULL) {
+	
+		/* count the number of uris with local paths */
+		count = 0;
+		total = g_list_length ((GList *) parameter_uris);
+		for (p = parameter_uris; p != NULL; p = p->next) {
+			local_path = gnome_vfs_get_local_path_from_uri ((const char *) p->data);
+			if (local_path != NULL) {
+				g_free (local_path);
+				count++;
+			}
+		}
+
+		if (count == 0) {
+			/* all files are non-local */
+			eel_show_error_dialog
+				(_("This drop target only supports local files.\n\n"
+				   "To open non-local files copy them to a local folder and then"
+				   " drop them again."),
+				 _("Drop target only supports local files"),
+				 parent_window);
+			
+			gnome_desktop_item_unref (ditem);
+			return;
+
+		} else if (count != total) {
+			/* some files were non-local */
+			eel_show_warning_dialog
+				(_("This drop target only supports local files.\n\n"
+				   "To open non-local files copy them to a local folder and then"
+				   " drop them again. The local files you dropped have already been opened."),
+				 _("Drop target only supports local files"),
+				 parent_window);
+		}		
+	}
+	
+	error = NULL;
+	gnome_desktop_item_launch (ditem, (GList *) parameter_uris,
+				   GNOME_DESKTOP_ITEM_LAUNCH_APPEND_URIS,
+				   &error);
+
+	if (error != NULL) {
+		message = g_strconcat (_("There was an error launching the application.\n\n"
+					 "Details: "), error->message, NULL);
+		eel_show_error_dialog
+			(message,
+			 _("Error launching application"),
+			 parent_window);			
+			 
+		g_error_free (error);
+		g_free (message);
+	}
+	
+	gnome_desktop_item_unref (ditem);
 }
