@@ -35,6 +35,7 @@
 #include "nautilus-window-private.h"
 #include "nautilus-zoom-control.h"
 #include <bonobo/bonobo-ui-util.h>
+#include <eel/eel-accessibility.h>
 #include <eel/eel-debug.h>
 #include <eel/eel-gdk-extensions.h>
 #include <eel/eel-glib-extensions.h>
@@ -44,6 +45,7 @@
 #include <eel/eel-vfs-extensions.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
+#include <gdk/gdkx.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-dialog-util.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
@@ -608,12 +610,6 @@ ref_now_unref_at_idle_time (GObject *object)
         g_idle_add (unref_callback, object);
 }
 
-static gboolean
-use_saved_window_positions (void)
-{
-        return eel_preferences_get_boolean (NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW);
-}
-
 /* This is called when we have decided we can actually change to the new view/location situation. */
 static void
 location_has_really_changed (NautilusWindow *window)
@@ -658,8 +654,7 @@ location_has_really_changed (NautilusWindow *window)
          * metadata. Then tell the callback it needs to show the
          * window
          */
-        if (!use_saved_window_positions () ||
-            window->show_state == NAUTILUS_WINDOW_POSITION_SET) {
+        if (window->show_state == NAUTILUS_WINDOW_POSITION_SET) {
                 gtk_widget_show (GTK_WIDGET (window));
         } else {
                 window->show_state = NAUTILUS_WINDOW_SHOULD_SHOW;
@@ -980,6 +975,10 @@ load_content_view (NautilusWindow *window,
                 /* create a new content view */
                 view = nautilus_view_frame_new (window->details->ui_container,
                                                 window->application->undo_manager);
+
+                eel_accessibility_set_name (view, _("Content View"));
+                eel_accessibility_set_description (view, _("View of the current file or folder"));
+                
                 window->new_content_view = view;
                 g_object_ref (view);
                 gtk_object_sink (GTK_OBJECT (view));
@@ -1115,27 +1114,61 @@ cancel_location_change (NautilusWindow *window)
         end_location_change (window);
 }
 
+
+static gboolean
+pending_location_already_showing (NautilusWindow *window)
+{
+	char *temp;
+	char *location;
+	GList *list, *item;
+	
+	temp = window->details->pending_location;
+	list = nautilus_application_get_window_list ();
+	for (item = list; item != NULL; item = item->next) {
+		location = nautilus_window_get_location (NAUTILUS_WINDOW (item->data));
+
+		if (!NAUTILUS_IS_DESKTOP_WINDOW (item->data)
+		    && location != NULL
+		    && item->data != window
+		    && !strcmp (temp, location)) {
+		    	g_free (location);
+			return TRUE;
+		}
+		
+		g_free (location);
+	}
+	
+	return FALSE;
+}
+
+
 static void
 position_and_show_window_callback (NautilusFile *file,
                        		   gpointer callback_data)
 {
 	NautilusWindow *window;
 	char *geometry_string;
-        
+   
 	window = NAUTILUS_WINDOW (callback_data);
 
-	if (use_saved_window_positions ()) {
-		geometry_string = nautilus_file_get_metadata 
+        if (!NAUTILUS_IS_DESKTOP_WINDOW (window)) {
+                /* load the saved window geometry */
+                geometry_string = nautilus_file_get_metadata 
 			(file, NAUTILUS_METADATA_KEY_WINDOW_GEOMETRY, NULL);
-		if (geometry_string != NULL) {
-			eel_gtk_window_set_initial_geometry_from_string 
-				(GTK_WINDOW (window), 
-				 geometry_string,
-				 NAUTILUS_WINDOW_MIN_WIDTH, 
-				 NAUTILUS_WINDOW_MIN_HEIGHT);
-		}
-		g_free (geometry_string);
-	}
+                if (geometry_string != NULL) {
+			/* Ignore saved window position if a window with the same
+			 * location is already showing. That way the two windows
+			 * wont appear at the exact same location on the screen.
+			 */
+                        eel_gtk_window_set_initial_geometry_from_string 
+                                (GTK_WINDOW (window), 
+                                 geometry_string,
+                                 NAUTILUS_WINDOW_MIN_WIDTH, 
+                                 NAUTILUS_WINDOW_MIN_HEIGHT,
+				 pending_location_already_showing (window));
+                }
+                g_free (geometry_string);
+        }
 
         /* If we finished constructing the window by now we need
          * to show the window here.
@@ -1190,22 +1223,20 @@ determined_initial_view_callback (NautilusDetermineViewHandle *handle,
 		 * windows), position and show it only after we've got the
 		 * metadata (since position info is stored there).
 		 */
-                if (use_saved_window_positions ()) {
-                        window->show_state = NAUTILUS_WINDOW_NOT_SHOWN;
-                        if (!GTK_WIDGET_VISIBLE (window)) {
-                                file = nautilus_file_get (location);
+		window->show_state = NAUTILUS_WINDOW_NOT_SHOWN;
+		if (!GTK_WIDGET_VISIBLE (window)) {
+			file = nautilus_file_get (location);
                                 
-                                attributes = g_list_prepend (NULL, NAUTILUS_FILE_ATTRIBUTE_METADATA);
-                                nautilus_file_call_when_ready (file,
-                                                               attributes,
-                                                               position_and_show_window_callback,
-                                                               window);
-                                g_list_free (attributes);
-                        }
-                }
+			attributes = g_list_prepend (NULL, NAUTILUS_FILE_ATTRIBUTE_METADATA);
+			nautilus_file_call_when_ready (file,
+                                                       attributes,
+                                                       position_and_show_window_callback,
+                                                       window);
+			g_list_free (attributes);
+		}
 
-                load_content_view (window, initial_view);
-                return;
+		load_content_view (window, initial_view);
+		return;
         }
         
         /* Some sort of failure occurred. How 'bout we tell the user? */
@@ -1347,7 +1378,11 @@ determined_initial_view_callback (NautilusDetermineViewHandle *handle,
 			   better test for that */
 
                         if (!eel_uris_match (location, "file:///")) {
+#ifdef WEB_NAVIGATION_ENABLED
                                 home_uri = eel_preferences_get (NAUTILUS_PREFERENCES_HOME_URI);
+#else
+                                home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
+#endif
                                 if (!eel_uris_match (home_uri, location)) {	
                                         nautilus_window_go_home (NAUTILUS_WINDOW (window));
                                 } else {
@@ -1564,6 +1599,12 @@ nautilus_window_set_sidebar_panels (NautilusWindow *window,
                 /* Create and load the panel. */
 		sidebar_panel = nautilus_view_frame_new (window->details->ui_container,
                                                          window->application->undo_manager);
+                
+                eel_accessibility_set_name (sidebar_panel, _("Side Pane"));
+                eel_accessibility_set_description
+                        (sidebar_panel, _("Contains a side pane view"));
+                
+
 		nautilus_view_frame_set_label (sidebar_panel, identifier->name);
 		set_view_frame_info (sidebar_panel, TRUE, identifier);
 		connect_view (window, sidebar_panel);
